#define ALLOW_ACCESS_TO_INTERNALS_OF_FORMS
#define ALLOW_ACCESS_TO_INTERNALS_OF_WINDOWS
#include <PalmOS.h>
#include "Diagnoser.h"

#define RESID 1000

#define FTR_ACCESSORIZER_BUSY 5
#define accessorizerCreator   'Acce'

Err AiaLibOpen(UInt16 refnum)	SYS_TRAP(sysLibTrapOpen);

Err AiaLibClose(UInt16 refnum)	SYS_TRAP(sysLibTrapClose);

UInt16 AiaGetInputAreaState(UInt16 refnum)
				SYS_TRAP(sysLibTrapCustom + 17);

Err AiaSetInputAreaState(UInt16 refnum, UInt16 state)
				SYS_TRAP(sysLibTrapCustom + 18);

typedef struct {
    UInt32    depth;
    Boolean   enableColor;
    FormType* activeForm;
    WinHandle drawWin;
    WinHandle activeWin;
    WinHandle savedBits;
    UInt32    statBarVisible;
    UInt16    orientation;
    UInt16    diaState;
    UInt16    trigger;
    Boolean   havePin;
    RGBColorType back;
    RGBColorType fore;
    RGBColorType text;
} WindowState;


void Go( void );

void DA( void )
{
    UInt32 value;


    if ( PrefGetPreference( prefDeviceLocked ) ||
         errNone == FtrGet( accessorizerCreator, FTR_ACCESSORIZER_BUSY, &value ) )
        return;
        
    FtrSet( accessorizerCreator, FTR_ACCESSORIZER_BUSY, 1 );

    Go();
    
    FtrUnregister( accessorizerCreator, FTR_ACCESSORIZER_BUSY );
}


Boolean EventHandler( EventType* event )
{
//    if ( event->eType == frmCloseEvent )
//        return false;
    return true;
}


Boolean IsModel( UInt32 manufacturer, UInt32 model )
{
    Err err;
    UInt32 x;
     
    if ( manufacturer != 0 ) {
         err = FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &x);
         if ( err != errNone || x != manufacturer )
             return false;
    }

    if ( model != 0 ) {
         err = FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &x);
         if ( err != errNone || x != model )
             return false;
    }

    return true;
}


Boolean SupportOrientation( void )
{
     Err err;
     UInt32 version;

     err = FtrGet( pinCreator, pinFtrAPIVersion, &version );

     if ( err != errNone || version == 0 )
         return false;

     if ( pinAPIVersion1_1 <= version )
         return true;

     if ( IsModel( 'Palm', 'Arz1' ) || IsModel( 'Tpwv', 0 ) )
         return true;

     return false;
}



Boolean FindApp( UInt16* card, LocalID* id, UInt16* launchCodeP )
{
    DmSearchStateType state;

    *launchCodeP = sysAppLaunchCmdCustomBase;

    return errNone == DmGetNextDatabaseByTypeCreator( true, &state,
               'appl', 'MyMe',
               false, card, id );
}



WinHandle SafeWindow( WinHandle winH )
{
    if ( winH != NULL )
        return winH;
    else
        return WinGetDisplayWindow();
}



Boolean IsPalmPDF( void )
{
    Err    err;
    UInt32 creator;

    return errNone == FtrGet( 'T3sk', 1051, &creator ) &&
         creator == 'PPDF';
}


Boolean SaveState( WindowState* s )
{
    Err err;
    RectangleType rect;
    UInt32        version;
    UInt32        creator;
    
    WinSetTextColorRGB( NULL, &( s->text ) );
    WinSetBackColorRGB( NULL, &( s->back ) );
    WinSetForeColorRGB( NULL, &( s->fore ) );

    WinScreenMode( winScreenModeGet, NULL, NULL, &( s->depth ), &( s->enableColor ) );

    err = FtrGet( pinCreator, pinFtrAPIVersion, &version );

    if ( err == errNone ) {
        if ( SupportOrientation() )
            s->orientation = SysGetOrientation();
        StatGetAttribute( statAttrBarVisible, &s->statBarVisible );
        s->diaState = PINGetInputAreaState();
        s->trigger  = PINGetInputTriggerState();
        if ( s->trigger && s->diaState == pinInputAreaClosed )
            return false;
        s->havePin = true;
    }
    else {
        s->havePin = false;
    }

    s->drawWin    = WinGetDrawWindow();
    s->activeWin  = WinGetActiveWindow();
    s->activeForm = FrmGetActiveForm();

    if ( IsPalmPDF() ) {
        WinSetDrawWindow( WinGetDisplayWindow() );
        WinGetDrawWindowBounds( &rect );
        rect.topLeft.x = 0;
        rect.topLeft.y = 0;

        s->savedBits   = WinSaveBits( &rect, &err );

        WinSetDrawWindow( SafeWindow( s->drawWin ) );
    
        return s->savedBits != NULL;
    }
    else {
        s->savedBits = NULL;

        return true;
    }
}



void RestoreState( WindowState* s )
{
    if ( s->activeForm != NULL ) {
        FrmSetActiveForm( s->activeForm );
    }
    else {
        WinSetActiveWindow( SafeWindow( s->activeWin ) );
    }

    if ( s->savedBits != NULL ) {
        WinSetDrawWindow( WinGetDisplayWindow() );
        WinRestoreBits( s->savedBits, 0, 0 );
    
        WinSetDrawWindow( SafeWindow( s->drawWin ) );
    }    

    WinSetTextColorRGB( &( s->text ), NULL );
    WinSetBackColorRGB( &( s->back ), NULL );
    WinSetForeColorRGB( &( s->fore ), NULL );

}



void RestoreDIA( WindowState* s )
{
    if ( s->havePin ) {
        UInt32 visible;

    LOCATION( 10 );
        if ( SupportOrientation() &&
             SysGetOrientation() != s->orientation )
            SysSetOrientation( s->orientation );

    LOCATION( 13 );
        if ( PINGetInputAreaState() != s->diaState ) {
    LOCATION( 14 );
            PINSetInputAreaState( s->diaState );
        }

    LOCATION( 11 );
        StatGetAttribute( statAttrBarVisible, &visible );
        if ( ! visible != ! s->statBarVisible ) {
    LOCATION( 12 );
            if ( s->statBarVisible ) {
                StatShow();
            }
            else {
                StatHide();
            }
        }

    LOCATION( 15 );
        if ( PINGetInputTriggerState() != s->trigger ) {
    LOCATION( 16 );
            PINSetInputTriggerState( s->trigger );
        }
    }
    LOCATION( 17 );
}


void RestoreMode( WindowState* s )
{
    UInt32 depth;
    Boolean color;
    
    WinScreenMode( winScreenModeGet, NULL, NULL, &depth, &color );

    if ( depth != s->depth )
        WinScreenMode( winScreenModeSet, NULL, NULL, &( s->depth ), NULL );

    if ( color != s->enableColor )
        WinScreenMode( winScreenModeSet, NULL, NULL, NULL, &( s->enableColor ) );
}


Err DisableNotification( SysNotifyParamType *np )
{
    /* disable notification */
    np->handled    = 0xFF;
    np->notifyType = 0;
    return errNone;
}


void Register( UInt32 type, SysNotifyProcPtr f, Int8 priority )
{
    UInt16    card;
    LocalID   id;

    if ( errNone == SysCurAppDatabase( &card, &id ) )
        SysNotifyRegister( card, id, type, f, priority, NULL );
}


void Unregister( UInt32 type, Int8 priority )
{
    UInt16    card;
    LocalID   id;

    if ( errNone == SysCurAppDatabase( &card, &id ) )
        SysNotifyUnregister( card, id, type, priority );
    else {
    }
}


static void InputMode( UInt16 mode )
{
      UInt16 refNum;

      if ( errNone == SysLibLoad( sysFileTLibrary, 'pinM', &refNum ) ) {
          if ( errNone == AiaLibOpen( refNum ) ) {
              AiaSetInputAreaState( refNum, mode );
              AiaLibClose( refNum );
          }
          SysLibRemove( refNum );
      }
}


void Go( void )
{
    UInt16 card;
    LocalID id;
    UInt16  formID;
    UInt32  result;
    FormType* newForm;
    RectangleType rect;
    Char          zero;
    UInt16        launchCode;
    EventType     e;
    WindowState   state;
    WindowState   state0;
    RectangleType zeroClip;

    if ( ! IsPalmPDF() && ! SaveState( &state0 ) )
        return;

    WinGetDisplayExtent( &rect.extent.x, &rect.extent.y );

    if ( ! IsPalmPDF() && ( rect.extent.y != 160 || rect.extent.x != 160 ) ) {
        InputMode( 1 );
        WinGetDisplayExtent( &rect.extent.x, &rect.extent.y );
    }

//    if ( rect.extent.x != 160 || rect.extent.y != 160 )
//        return;

    if ( ! SaveState( &state ) )
        return;

    WinSetDrawWindow( WinGetDisplayWindow() );

    zero = 0;

    if ( FrmGetActiveFormID() == 41234 )
        formID = 51284;
    else
        formID = 41234;

    newForm = FrmNewForm( formID, &zero, 0, 0, 160, 160, false, 0, 0, 0 );

    FrmSetActiveForm( newForm );

    FrmSetEventHandler( newForm, EventHandler );

    InputMode( 3 );
    FrmDrawForm( newForm );

    rect.extent.x  = 160;
    rect.extent.y  = 160;
    rect.topLeft.x = 0;
    rect.topLeft.y = 0;

    WinSetDrawWindow( WinGetDisplayWindow() );
    WinEraseRectangle( &rect, 0 );

    if ( FindApp( &card, &id, &launchCode ) )
        SysAppLaunch( card, id,
            /* sysAppLaunchFlagUIApp | */
            sysAppLaunchFlagNewStack | sysAppLaunchFlagNewGlobals,
            launchCode, NULL, &result );

    LOCATION( 0 );
//    RestoreMode( &state );
    LOCATION( 1 );

    RestoreDIA( &state );

    MemSet( &zeroClip, sizeof( zeroClip ), 0 );
    WinSetDrawWindow( FrmGetWindowHandle( newForm ) );
    WinSetClip( &zeroClip );

    LOCATION( 2 );
    FrmEraseForm( newForm );
    LOCATION( 3 );
    FrmDeleteForm( newForm );
    LOCATION( 4 );

    RestoreState( &state );
    if ( ! IsPalmPDF() )
        RestoreDIA( &state0 );

//    Unregister( sysNotifyDisplayResizedEvent, -0x80 );

    EvtGetEvent( &e, 0 );
    SysHandleEvent( &e );
    EvtGetEvent( &e, 0 );
    SysHandleEvent( &e );
    EvtGetEvent( &e, 0 );
    SysHandleEvent( &e );
}

